﻿import es
import gamethread
import keyvalues
import langlib
import playerlib
import popuplib

lang_dir = "cstrike/addons/eventscripts/soccer_mod_2011/language/"

messages_common = langlib.Strings(lang_dir + "messages_common.ini")
messages_text = langlib.Strings(lang_dir + "admin/messages_admin.ini")
popup_common = langlib.Strings(lang_dir + "popup_common.ini")
popup_text = langlib.Strings(lang_dir + "admin/popup_admin.ini")

# ******************** Load ********************
def load():
	es.keygroupcreate("sm_keygroup_set")

	create_popup()

	es.load("soccer_mod_2011/admin/manager")
	es.load("soccer_mod_2011/admin/settings")

# ******************** Create popup ********************
def create_popup():
	popup = popuplib.create("sm_popup_admin")

	popup.addline("SOCCER MOD %s" % (popup_text("title")))
	popup.addline(" ")

	number = 1

	if es.ServerVar("sm_module_match_on") == 1:
		popup.addline("->%s. %s" % (number, popup_text("match")))
		popup.select(number, match)
		number += 1

	if es.ServerVar("sm_module_cap_on") == 1:
		popup.addline("->%s. %s" % (number, popup_text("cap")))
		popup.select(number, cap)
		number += 1

	if es.ServerVar("sm_module_referee_on") == 1:
		popup.addline("->%s. %s" % (number, popup_text("referee")))
		popup.select(number, referee)
		number += 1

	if es.ServerVar("sm_module_training_on") == 1:
		popup.addline("->%s. %s" % (number, popup_text("training")))
		popup.select(number, training)
		number += 1

	if es.ServerVar("sm_module_skins_on") == 1:
		popup.addline("->%s. %s" % (number, popup_text("skins")))
		popup.select(number, skins)
		number += 1

	popup.addline("->%s. %s" % (number, popup_text("map")))
	popup.select(number, map)
	number += 1

	popup.addline("->%s. %s" % (number, popup_text("settings")))
	popup.select(number, settings)
	number += 1

	popup.addline("->%s. %s" % (number, popup_text("manager")))
	popup.select(number, manager)
	number += 1

	popup.addline(" ")
	popup.addline("0. %s" % (popup_common("close")))

	while number < 10:
		popup.submenu(number, "sm_popup_admin")
		number += 1

# ******************** Events ********************
def es_map_start(event_var):
	es.keygroupdelete("sm_keygroup_set")
	es.keygroupcreate("sm_keygroup_set")

def player_disconnect(event_var):
	if es.exists("key", "sm_keygroup_set", event_var["steamid"]) == 1:
		es.keydelete("sm_keygroup_set", event_var["steamid"])

	name = "sm_popup_%s" % (event_var["userid"])
	if popuplib.exists(name):
		popuplib.delete(name)

# ******************** Commands ********************
def cmd_admin():
	userid = es.getcmduserid()
	player = playerlib.getPlayer(userid)

	if es.exists("key", "soccer_mod_admins", player.steamid) == 1:
		popuplib.send("sm_popup_admin", userid)
	else:
		es.tell(userid, "#green", messages_text("admin", None, player.get("lang")))

# ******************** Popup choice ********************
def match(userid, choice, name):
	check_access("match", userid)

def cap(userid, choice, name):
	check_access("cap", userid)

def referee(userid, choice, name):
	check_access("referee", userid)

def training(userid, choice, name):
	check_access("training", userid)

def skins(userid, choice, name):
	check_access("skins", userid)

def map(userid, choice, name):
	player = playerlib.getPlayer(userid)

	try:
		if int(es.keygetvalue("soccer_mod_admins", player.steamid, "change_map")) == 1:
			if es.ServerVar("sm_match_started") == 0:
				popup_name = "sm_popup_%s" % (userid)
				if popuplib.exists(popup_name):
					popuplib.delete(popup_name)

				myPopup = popuplib.easymenu(popup_name, None, change_map)
				myPopup.settitle(messages_text("change_map"))

				maps = keyvalues.getKeyGroup("soccer_mod_maps")
				for item in maps:
					myPopup.addoption(item, item)

				myPopup.submenu(10, "sm_popup_admin")

				popuplib.send(popup_name, userid)
			else:
				es.tell(userid, "#green", messages_common("no_option_match", None, player.get("lang")))

				popuplib.send("sm_popup_admin", userid)
		else:
			es.tell(userid, "#green", messages_text("menu", None, player.get("lang")))

			popuplib.send("sm_popup_admin", userid)
	except ValueError:
		es.tell(userid, "#green", messages_text("menu", None, player.get("lang")))

		popuplib.send("sm_popup_admin", userid)

def settings(userid, choice, popupid):
	check_access("settings", userid)

def manager(userid, choice, popupid):
	player = playerlib.getPlayer(userid)

	try:
		if int(es.keygetvalue("soccer_mod_admins", player.steamid, "manager")) == 1:
			popuplib.send("sm_popup_admin_manager", userid)
		else:
			es.tell(userid, "#green", messages_text("menu", None, player.get("lang")))

			popuplib.send("sm_popup_admin", userid)
	except ValueError:
		es.tell(userid, "#green", messages_text("menu", None, player.get("lang")))

		popuplib.send("sm_popup_admin", userid)

# ******************** Easymenu choice ********************
def change_map(userid, choice, name):
	player = playerlib.getPlayer(userid)

	if es.ServerVar("sm_match_started") == 0:
		tokens = {}

		if es.exists("map", choice) == 1:
			tokens["name"] = player.name
			tokens["map"] = choice
			lang_msg("changed_map", tokens)

			gamethread.delayed(3, es.server.queuecmd, ("changelevel %s" % (choice)))
		else:
			tokens["name"] = choice
			es.tell(userid, "#green", messages_text("map_no_exist", tokens, player.get("lang")))

			popuplib.send("sm_popup_admin", userid)
	else:
		es.tell(userid, "#green", messages_text("no_change_map_match", None, player.get("lang")))

		popuplib.send("sm_popup_admin", userid)

# ******************** Functions ********************
def lang_msg(text, tokens):
	for player in playerlib.getPlayerList("#human"):
		es.tell(player.userid, "#green", messages_text(text, tokens, player.get("lang")))

def check_access(module, userid):
	player = playerlib.getPlayer(userid)

	if es.ServerVar("soccer_mod_allowed") == 1:
		try:
			if int(es.keygetvalue("soccer_mod_admins", player.steamid, module)) == 1:
				popuplib.send("sm_popup_admin_%s" % (module), userid)
			else:
				es.tell(userid, "#green", messages_text("menu", None, player.get("lang")))

				popuplib.send("sm_popup_admin", userid)
		except ValueError:
			es.tell(userid, "#green", messages_text("menu", None, player.get("lang")))

			popuplib.send("sm_popup_admin", userid)
	else:
		es.tell(userid, "#green", messages_common("not_allowed", None, player.get("lang")))

		popuplib.send("sm_popup_admin", userid)

# ******************** Unload ********************
def unload():
	es.keygroupdelete("sm_keygroup_set")

	popuplib.delete("sm_popup_admin")

	es.unload("soccer_mod_2011/admin/manager")
	es.unload("soccer_mod_2011/admin/settings")