﻿import es
import gamethread
import langlib
import playerlib
import popuplib

lang_dir = "cstrike/addons/eventscripts/soccer_mod_2011/language/"

messages_common = langlib.Strings(lang_dir + "messages_common.ini")
messages_text = langlib.Strings(lang_dir + "modules/referee/messages_referee_warn.ini")
popup_common = langlib.Strings(lang_dir + "popup_common.ini")
popup_text = langlib.Strings(lang_dir + "modules/referee/popup_admin_referee_warn.ini")

# ******************** Load ********************
def load():
	popup = popuplib.create("sm_popup_admin_referee_warn")

	popup.addline("SOCCER MOD %s" % (popup_text("title")))
	popup.addline(" ")

	tokens = {}
	for i in range(1, 8):
		tokens["warning"] = es.ServerVar("sm_referee_warning_%s" % (i))
		popup.addline("->%s. %s" % (i, popup_text("warning", tokens)))

	popup.addline("->8. %s" % (popup_text("custom")))
	popup.addline(" ")
	popup.addline("->9. %s" % (popup_common("back")))
	popup.addline(" ")
	popup.addline("->0. %s" % (popup_common("close")))

	for i in range(1, 9):
		popup.select(i, popup_choice)

	popup.submenu(9, "sm_popup_admin_referee")

	es.addons.registerSayFilter(filter)

# ******************** Say filter ********************
def filter(userid, text, teamonly):
	steamid = es.getplayersteamid(userid)

	if es.exists("key", "sm_keygroup_set", steamid) == 1:
		type = es.keygetvalue("sm_keygroup_set", steamid, "setting_type")
		text = text.strip('"')

		if type == "referee_custom_warning":
			if text != "!cancel":
				warn_player(userid, es.keygetvalue("sm_keygroup_set", steamid, "userid"), text)
			else:
				popuplib.send("sm_popup_admin_referee_warn", userid)

			es.keydelete("sm_keygroup_set", steamid)
			return (0, None, 0)
		else:
			return (userid, text, teamonly)
	else:
		return (userid, text, teamonly)

# ******************** Popup choice ********************
def popup_choice(userid, choice, name):
	popup_name = "sm_popup_%s" % (userid)
	if popuplib.exists(popup_name):
		popuplib.delete(popup_name)

	myPopup = popuplib.easymenu(popup_name, None, select_player)
	myPopup.settitle(messages_text("select_player"))

	if choice < 8:
		type = "warning_%s" % (choice)
	elif choice == 8:
		type = "custom_warning"

	for player in playerlib.getPlayerList("#human"):
		myPopup.addoption("%s_%s" % (type, player.userid), player.name)

	myPopup.submenu(10, "sm_popup_admin_referee_warn")

	popuplib.send(popup_name, userid)

# ******************** Easymenu choice ********************
def select_player(userid, choice, name):
	type = choice.split("_")
	if type[0] == "warning":
		warn_player(userid, type[2], es.ServerVar("sm_referee_warning_%s" %(type[1])))
	elif type[0] == "custom":
		tokens = {}
		player = playerlib.getPlayer(userid)

		popup_name = "sm_popup_%s" % (userid)
		if popuplib.exists(popup_name):
			popuplib.delete(popup_name)

		if es.exists("userid", type[2]) == 1:
			if es.exists("key", "sm_keygroup_set", player.steamid) == 0:
				es.keycreate("sm_keygroup_set", player.steamid)

			es.keysetvalue("sm_keygroup_set", player.steamid, "setting_type", "referee_custom_warning")
			es.keysetvalue("sm_keygroup_set", player.steamid, "userid", type[2])

			es.tell(userid, "#green", messages_text("set_warning", tokens, player.get("lang")))
		else:
			es.tell(userid, "#green", messages_common("no_player_found", tokens, player.get("lang")))
			popuplib.send("sm_popup_admin_referee_warn", userid)

# ******************** Functions ********************
def lang_msg(text, tokens):
	for player in playerlib.getPlayerList("#human"):
		es.tell(player.userid, "#green", messages_text(text, tokens, player.get("lang")))

def warn_player(userid, choice, warning):
	tokens = {}
	player = playerlib.getPlayer(userid)

	popup_name = "sm_popup_%s" % (userid)
	if popuplib.exists(popup_name):
		popuplib.delete(popup_name)

	if es.exists("userid", choice) == 1:
		target = playerlib.getPlayer(choice)

		es.cexec_all("playgamesound", es.ServerVar("sm_referee_sound_whistle"))

		tokens["name"] = player.name
		tokens["target"] = target.name
		tokens["warning"] = warning
		lang_msg("warn_message", tokens)

		es.menu(5, int(choice), messages_text("warning", tokens, target.get("lang")))

		es.centertell(choice, messages_text("warning", tokens, target.get("lang")))
		gamethread.delayed(1, es.centertell, (choice, messages_text("warning", tokens, target.get("lang"))))
		gamethread.delayed(2, es.centertell, (choice, messages_text("warning", tokens, target.get("lang"))))
		gamethread.delayed(3, es.centertell, (choice, messages_text("warning", tokens, target.get("lang"))))
		gamethread.delayed(4, es.centertell, (choice, messages_text("warning", tokens, target.get("lang"))))
	else:
		es.tell(userid, "#green", messages_common("no_player_found", tokens, player.get("lang")))

	popuplib.send("sm_popup_admin_referee_warn", userid)

# ******************** Unload ********************
def unload():
	es.addons.unregisterSayFilter(filter)

	popuplib.delete("sm_popup_admin_referee_warn")