﻿import es
import gamethread
import langlib
import math
import os
import playerlib
import popuplib
import random
import sqlite3
import time

lang_dir = "cstrike/addons/eventscripts/soccer_mod_2011/language/"

messages_common = langlib.Strings(lang_dir + "messages_common.ini")
messages_text = langlib.Strings(lang_dir + "modules/stats/messages_stats.ini")
popup_common = langlib.Strings(lang_dir + "popup_common.ini")
popup_text = langlib.Strings(lang_dir + "modules/stats/popup_stats.ini")

score_sheet = []

database = "cstrike/cfg/soccer_mod/ranking.db"

exists = os.path.isfile(database)

connection = sqlite3.connect(database)
cursor = connection.cursor()

if exists != True:
	table = "CREATE TABLE [players] ("
	table += "[steamid] VARCHAR(255)  PRIMARY KEY NOT NULL,"
	table += "[name] VARCHAR(255)  NOT NULL,"
	table += "[connected] VARCHAR(255)  NOT NULL,"
	table += "[goals] INTEGER DEFAULT '0' NOT NULL,"
	table += "[assists] INTEGER DEFAULT '0' NOT NULL,"
	table += "[own_goals] INTEGER DEFAULT '0' NOT NULL,"
	table += "[hits] INTEGER DEFAULT '0' NOT NULL,"
	table += "[passes] INTEGER DEFAULT '0' NOT NULL,"
	table += "[interceptions] INTEGER DEFAULT '0' NOT NULL,"
	table += "[losses] INTEGER DEFAULT '0' NOT NULL,"
	table += "[rank] INTEGER DEFAULT '0' NOT NULL,"
	table += "[time] INTEGER DEFAULT '0' NOT NULL)"
	cursor.execute(table)

exists = 0
cursor.execute("PRAGMA table_info(players)")
for row in cursor:
	for item in row:
		if item == "time":
			exists = 1

if exists == 0:
	cursor.execute("ALTER TABLE players ADD time INTEGER DEFAULT '0' NOT NULL")

# ******************** Load ********************
def load():
	load_variables()
	reset_team_stats()

	es.keygroupcreate("soccer_mod_stats")
	es.keygroupcreate("soccer_mod_time")

	cmd = "sm_stats_event_%s" % (random.randint(1, 99999))
	es.set("sm_stats_event", cmd)
	es.regclientcmd(cmd, "soccer_mod_2011/modules/stats/ball_hit")

	popup = popuplib.create("sm_popup_stats")

	popup.addline("SOCCER MOD %s" % (popup_text("title")))
	popup.addline(" ")
	popup.addline("->1. %s" % (popup_text("team")))
	popup.addline("->2. %s" % (popup_text("personal")))
	popup.addline("->3. %s" % (popup_text("player")))
	popup.addline("->4. %s" % (popup_text("ranking")))
	popup.addline("->5. %s" % (popup_text("score_sheet")))
	popup.addline(" ")
	popup.addline("0. %s" % (popup_common("close")))

	popup.select(1, team)
	popup.select(2, personal)
	popup.select(3, player)
	popup.submenu(4, "sm_popup_stats_ranking")
	popup.select(5, score_sheet_choice)

	for i in range(6, 10):
		popup.submenu(i, "sm_popup_stats")

	popup = popuplib.create("sm_popup_stats_team")

	popup.addline("SOCCER MOD %s" % (popup_text("title_team")))
	popup.addline(" ")
	popup.addline("Home Team:")
	popup.addline(" ")
	popup.addline("Possession")
	popup.addline("Goals")
	popup.addline("Assists")
	popup.addline("Own goals")
	popup.addline("Hits")
	popup.addline("Passes")
	popup.addline("Interceptions")
	popup.addline("Losses")
	popup.addline(" ")
	popup.addline("Away Team:")
	popup.addline(" ")
	popup.addline("Possession")
	popup.addline("Goals")
	popup.addline("Assists")
	popup.addline("Own goals")
	popup.addline("Hits")
	popup.addline("Passes")
	popup.addline("Interceptions")
	popup.addline("Losses")
	popup.addline(" ")
	popup.addline("->9. %s" % (popup_common("back")))
	popup.addline(" ")
	popup.addline("0. %s" % (popup_common("close")))

	for i in range(1, 9):
		popup.select(i, team)

	popup.submenu(9, "sm_popup_stats")

	reset_score_sheet()

	es.load("soccer_mod_2011/modules/stats/ranking")

# ******************** Load variables ********************
def load_variables():
	global scorer
	scorer = 0

	global scorer_time_stamp
	scorer_time_stamp = 0

	global assister
	assister = 0

	global assister_time_stamp
	assister_time_stamp = 0

def reset_team_stats():
	stats = [
		"sm_stats_pos_t",
		"sm_stats_pos_ct",
		"sm_stats_total_pos",
		"sm_stats_number_goals_t",
		"sm_stats_number_goals_ct",
		"sm_stats_number_assists_t",
		"sm_stats_number_assists_ct",
		"sm_stats_number_own_goals_t",
		"sm_stats_number_own_goals_ct",
		"sm_stats_number_hits_t",
		"sm_stats_number_hits_ct",
		"sm_stats_number_passes_t",
		"sm_stats_number_passes_ct",
		"sm_stats_number_interceptions_t",
		"sm_stats_number_interceptions_ct",
		"sm_stats_number_losses_t",
		"sm_stats_number_losses_ct"
	]

	for item in stats:
		es.set(item, 0)

# ******************** Events ********************
def es_map_start(event_var):
	connection.commit()

	load_variables()
	reset_team_stats()
	reset_score_sheet()

	es.keygroupdelete("soccer_mod_stats")
	es.keygroupcreate("soccer_mod_stats")

def player_activate(event_var):
	if es.ServerVar("soccer_mod_allowed") == 1:
		player = playerlib.getPlayer(event_var["userid"])
		if player.steamid != "BOT":
			set_connected(player.steamid, player.name)

			if es.exists("key", "soccer_mod_time", player.steamid) == 0:
				es.keycreate("soccer_mod_time", player.steamid)

			es.keysetvalue("soccer_mod_time", player.steamid, "time", int(time.time()))

def player_disconnect(event_var):
	if es.ServerVar("soccer_mod_allowed") == 1:
		if len(playerlib.getPlayerList("#human")) < 1:
			connection.commit()

		if event_var["es_steamid"] != "BOT":
			set_connected(event_var["es_steamid"])

			if es.exists("key", "soccer_mod_time", event_var["es_steamid"]) == 1:
				seconds = int(time.time()) - int(es.keygetvalue("soccer_mod_time", event_var["es_steamid"], "time"))
				execute_query("UPDATE players SET time = (time + %s) WHERE steamid = '%s'" % (seconds, event_var["es_steamid"]))

				es.keydelete("soccer_mod_time", event_var["es_steamid"])

def player_spawn(event_var):
	if es.ServerVar("soccer_mod_allowed") == 1:
		if int(event_var["es_userteam"]) > 1:
			if es.exists("key", "soccer_mod_stats", event_var["es_steamid"]) == 1:
				es.server.queuecmd("score set %s %s" % (event_var["userid"], es.keygetvalue("soccer_mod_stats", event_var["es_steamid"], "goals")))
			else:
				es.server.queuecmd("score set %s 0" % (event_var["userid"]))

def round_start(event_var):
	if es.ServerVar("soccer_mod_allowed") == 1:
		es.set("sm_match_first_hit", 1)
		es.set("sm_stats_round_end", 0)
		es.set("sm_stats_goal_scored", 0)

		load_variables()

		players = playerlib.getUseridList("#all")
		if len(players) > 0:
			userid = random.choice(players)

			es.server.queuecmd("es_xgive %s point_clientcommand" % (userid))

			event = es.ServerVar("sm_stats_event")

			es.server.queuecmd('es_xfire %s func_physbox addoutput "OnDamaged point_clientcommand,command,%s"' % (userid, event))
			es.server.queuecmd('es_xfire %s prop_physics addoutput "OnTakeDamage point_clientcommand,command,%s"' % (userid, event))
			es.server.queuecmd('es_xfire %s trigger_hurt addoutput "OnStartTouch point_servercommand,command,es_xset sm_stats_goal_scored 1"' % (userid))

def round_end(event_var):
	if es.ServerVar("soccer_mod_allowed") == 1:
		es.set("sm_match_first_hit", 1)
		gamethread.delayed(0.15, show_stats, (int(event_var["winner"])))

# ******************** Commands ********************
def cmd_stats():
	userid = es.getcmduserid()
	player = playerlib.getPlayer(userid)

	if es.ServerVar("soccer_mod_allowed") == 1:
		popuplib.send("sm_popup_stats", userid)
	else:
		es.tell(userid, "#green", messages_common("not_allowed", None, player.get("lang")))

def ball_hit():
	if es.ServerVar("sm_stats_round_end") == 0:
		es.set("sm_match_first_hit", 0)

		global scorer
		temp_scorer = scorer
		scorer = playerlib.getPlayer(es.getcmduserid())

		if es.exists("key", "soccer_mod_stats", scorer.steamid) == 0:
			es.keycreate("soccer_mod_stats", scorer.steamid)

			values = ["possession", "goals", "assists", "own_goals", "hits", "passes", "interceptions", "losses"]
			for item in values:
				es.keysetvalue("soccer_mod_stats", scorer.steamid, item, 0)

		add_stat(scorer, "hits", es.ServerVar("sm_stats_rank_hit"))

		if int(scorer) != int(temp_scorer):
			global scorer_time_stamp
			global assister_time_stamp
			assister_time_stamp = scorer_time_stamp
			scorer_time_stamp = time.time()

			if int(temp_scorer) > 0:
				global assister
				assister = temp_scorer

				possession = round(scorer_time_stamp - assister_time_stamp, 1)

				if scorer.team == assister.team:
					#es.msg("%s passed the ball to %s." % (assister.name, scorer.name))
					add_stat(assister, "passes", es.ServerVar("sm_stats_rank_pass"), possession)
				else:
					#es.msg("%s intercepted the ball from %s." % (scorer.name, assister.name))
					add_stat(scorer, "interceptions", es.ServerVar("sm_stats_rank_interception"), possession)

					#es.msg("%s lost the ball to %s." % (assister.name, scorer.name))
					add_stat(assister, "losses", es.ServerVar("sm_stats_rank_loss"), possession)

				if possession > 0:
					#es.msg("%s has had the ball for %s seconds." % (assister.name, possession))
					add_possession(assister, possession)

# ******************** Popup choice ********************
def team(userid, choice, name):
	values = ["goals", "assists", "own_goals", "hits", "passes", "interceptions", "losses"]

	if es.ServerVar("sm_module_match_on") == 1:
		popuplib.find("sm_popup_stats_team").modline(3, "%s:" % (es.ServerVar("sm_match_name_t")))
	else:
		popuplib.find("sm_popup_stats_team").modline(3, "Terrorists:")

	possession = float(es.ServerVar("sm_stats_total_pos"))
	if possession > 0:
		value = float(es.ServerVar("sm_stats_pos_t"))
		percentage = str(int(round(((value / possession) * 100), 0))) + "%"
	else:
		percentage = "0%"

	popuplib.find("sm_popup_stats_team").modline(5, "%s: %s" % (popup_text("possession"), percentage))

	cvars = [
		"sm_stats_number_goals_t", "sm_stats_number_assists_t", "sm_stats_number_own_goals_t", "sm_stats_number_hits_t", 
		"sm_stats_number_passes_t", "sm_stats_number_interceptions_t", "sm_stats_number_losses_t"
	]
	for i in range(0, len(values)):
		popuplib.find("sm_popup_stats_team").modline(i + 6, "%s: %s" % (popup_text(values[i]), es.ServerVar(cvars[i])))

	if es.ServerVar("sm_module_match_on") == 1:
		popuplib.find("sm_popup_stats_team").modline(14, "%s:" % (es.ServerVar("sm_match_name_ct")))
	else:
		popuplib.find("sm_popup_stats_team").modline(14, "Counter-terrorists:")

	possession = float(es.ServerVar("sm_stats_total_pos"))
	if possession > 0:
		value = float(es.ServerVar("sm_stats_pos_ct"))
		percentage = str(int(round(((value / possession) * 100), 0))) + "%"
	else:
		percentage = "0%"

	popuplib.find("sm_popup_stats_team").modline(16, "%s: %s" % (popup_text("possession"), percentage))

	cvars = [
		"sm_stats_number_goals_ct", "sm_stats_number_assists_ct", "sm_stats_number_own_goals_ct", "sm_stats_number_hits_ct", 
		"sm_stats_number_passes_ct", "sm_stats_number_interceptions_ct", "sm_stats_number_losses_ct"
	]
	for i in range(0, len(values)):
		popuplib.find("sm_popup_stats_team").modline(i + 17, "%s: %s" % (popup_text(values[i]), es.ServerVar(cvars[i])))

	popuplib.send("sm_popup_stats_team", userid)

def personal(userid, choice, name):
	tokens = {}
	player = playerlib.getPlayer(userid)

	if es.exists("key", "soccer_mod_stats", es.getplayersteamid(userid)) == 1:
		create_key_menu(userid, "title_personal")
	else:
		es.tell(userid, "#green", messages_text("no_stats", tokens, player.get("lang")))
		popuplib.send("sm_popup_stats", userid)

def player(userid, choice, name):
	name = "sm_popup_%s" % (userid)
	if popuplib.exists(name):
		popuplib.delete(name)

	myPopup = popuplib.easymenu(name, None, player_choice)
	myPopup.settitle(popup_text("select_player"))

	for player in playerlib.getPlayerList("#human"):
		myPopup.addoption(player.userid, player.name)

	myPopup.submenu(10, "sm_popup_stats")

	popuplib.send(name, userid)

def score_sheet_choice(userid, choice, name):
	tokens = {}
	player = playerlib.getPlayer(userid)

	if len(score_sheet) > 0:
		popuplib.send("sm_popup_stats_score_sheet_1", userid)
	else:
		es.tell(userid, "#green", messages_text("empty", tokens, player.get("lang")))
		popuplib.send("sm_popup_stats", userid)

# ******************** Easymenu choice ********************
def player_choice(userid, choice, name):
	tokens = {}
	player = playerlib.getPlayer(userid)

	if es.exists("key", "soccer_mod_stats", es.getplayersteamid(choice)) == 1:
		create_key_menu(userid, "title_player", choice)
	else:
		name = "sm_popup_%s" % (userid)
		if popuplib.exists(name):
			popuplib.delete(name)

		es.tell(userid, "#green", messages_text("no_stats", tokens, player.get("lang")))
		popuplib.send("sm_popup_stats", userid)

# ******************** Functions ********************
def lang_msg(text, tokens):
	for player in playerlib.getPlayerList("#human"):
		es.tell(player.userid, "#green", messages_text(text, tokens, player.get("lang")))

def set_connected(steamid, name = None):
	cursor.execute("SELECT steamid FROM players WHERE steamid = '%s'" % (steamid))

	check = 0
	for row in cursor:
		check += 1

	if check < 1:
		cursor.execute("INSERT INTO players (name, steamid, connected) VALUES ('Encoding error', '%s', '%s')" % (steamid, int(time.time())))

	if name != None:
		cursor.execute("UPDATE players SET name = ?, connected = ? WHERE steamid = ?", (name, int(time.time()), steamid))
	else:
		cursor.execute("UPDATE players SET connected = ? WHERE steamid = ?", (int(time.time()), steamid))

def execute_query(query, value = None):
	cursor.execute(query)

	if value != None:
		return cursor

def show_stats(winner):
	es.set("sm_stats_round_end", 1)

	if int(es.ServerVar("sm_stats_goal_scored")) == 1:
		global scorer
		global assister

		if int(scorer) > 0:
			tokens = {}

			current = time.time()
			global scorer_time_stamp
			possession = round(current - scorer_time_stamp, 1)

			if scorer.team == winner:
				tokens["name"] = scorer.name
				lang_msg("goal", tokens)

				add_stat(scorer, "goals", es.ServerVar("sm_stats_rank_goal"), possession)

				if int(assister) > 0:
					if assister.team == winner:
						tokens["name"] = assister.name
						lang_msg("assist", tokens)

						add_stat(assister, "assists", es.ServerVar("sm_stats_rank_assist"), possession)
			else:
				tokens["name"] = scorer.name
				lang_msg("own_goal", tokens)

				add_stat(scorer, "own_goals", es.ServerVar("sm_stats_rank_own_goal"), possession)

			if possession > 0:
				#es.msg("#green", "%s has had the ball for %s seconds." % (scorer.name, possession))
				add_possession(scorer, possession)

			score_sheet_add(scorer, assister, winner)

	load_variables()

def add_stat(player, type, points, possession = None):
	if player.team == 2:
		name = "sm_stats_number_%s_t" % (type)
	elif player.team == 3:
		name = "sm_stats_number_%s_ct" % (type)
	es.set(name, es.ServerVar(name) + 1)

	es.keysetvalue("soccer_mod_stats", player.steamid, type, int(es.keygetvalue("soccer_mod_stats", player.steamid, type)) + 1)

	if es.ServerVar("sm_match_started") == 1:
		points *= float(es.ServerVar("sm_stats_rank_multiplier"))

	if player.team == 2:
		team = 3
	else:
		team = 2
	ratio = round(float(es.getplayercount(team)) / float(es.getplayercount(player.team)), 2)
	#es.msg(ratio)

	points *= es.getplayercount(2) + es.getplayercount(3)

	if possession != None:
		points += possession

	points = int(round(points, 0))

	execute_query("UPDATE players SET %s = (%s + 1), rank = (rank + %s) WHERE steamid = '%s'" % (type, type, points, player.steamid))

	#es.msg("#green", "%s has earned %s points." % (player.name, points))

def add_possession(player, possession):
	if player.team == 2:
		es.set("sm_stats_pos_t", es.ServerVar("sm_stats_pos_t") + possession)
	elif player.team == 3:
		es.set("sm_stats_pos_ct", es.ServerVar("sm_stats_pos_ct") + possession)

	es.set("sm_stats_total_pos", es.ServerVar("sm_stats_total_pos") + possession)

	es.keysetvalue("soccer_mod_stats", player.steamid, "possession", float(es.keygetvalue("soccer_mod_stats", player.steamid, "possession")) + possession)

def create_key_menu(userid, title, choice = 1):
	name = "sm_popup_%s" % (userid)
	if popuplib.exists(name):
		popuplib.delete(name)

	popup = popuplib.create(name)

	popup.addline("SOCCER MOD %s" % (popup_text(title)))
	popup.addline(" ")

	if choice > 1:
		player_name = es.getplayername(choice)
		popup.addline("%s:" % (player_name))
		popup.addline(" ")

		steamid = es.getplayersteamid(choice)
	else:
		steamid = es.getplayersteamid(userid)

	possession = float(es.ServerVar("sm_stats_total_pos"))
	if possession > 0:
		value = float(es.keygetvalue("soccer_mod_stats", steamid, "possession"))
		percentage = str(int(round(((value / possession) * 100), 0))) + "%"
	else:
		percentage = "0%"

	popup.addline("%s: %s" % (popup_text("possession"), percentage))

	stats = ["goals", "assists", "own_goals", "hits", "passes", "interceptions", "losses"]
	count = 0
	for stat in stats:
		value = float(es.keygetvalue("soccer_mod_stats", steamid, stats[count]))
		value_t = float(es.ServerVar("sm_stats_number_%s_t" % (stats[count])))
		value_ct = float(es.ServerVar("sm_stats_number_%s_ct" % (stats[count])))

		total = value_t + value_ct
		if total > 0:
			percentage = value / total
			percentage *= 100
			number = round(percentage, 0)
			percentage = str(int(number)) + "%"
		else:
			percentage = "0%"

		value = int(value)
		total = int(total)

		popup.addline("%s: %s / %s (%s)" % (popup_text(stat), value, total, percentage))
		count += 1
		
	popup.addline(" ")
	popup.addline("->9. %s" % (popup_common("back")))
	popup.addline(" ")
	popup.addline("0. %s" % (popup_common("close")))

	for i in range(1, 9):
		popup.submenu(i, name)

	popup.submenu(9, "sm_popup_stats")

	popuplib.send(name, userid)

def create_score_sheet():
	number = 1
	page = 1
	result = 1
	length = float(len(score_sheet))

	number_pages = int(math.ceil(length / 5))

	popup = popuplib.create("sm_popup_stats_score_sheet_1")

	popup.addline("SOCCER MOD %s (1/%s)" % (popup_text("title"), number_pages))
	popup.addline(" ")

	for row in score_sheet:
		popup.addline(row)

		number += 1
		result += 1

		if number > 5 or result > length:
			number = 1

			popup.addline(" ")

			popup.addline("->1. Display in console")
			popup.select(1, score_sheet_console)

			for i in range(2, 8):
				popup.submenu(i, "sm_popup_stats_score_sheet_%s" % (page))

			if length > 5:
				popup.addline(" ")

			page -= 1
			if page > 0:
				popup.addline("->8. %s" % (popup_common("previous")))
				popup.submenu(8, "sm_popup_stats_score_sheet_%s" % (page))

			page += 2
			if page <= number_pages:
				popup.addline("->9. %s" % (popup_common("next")))
				popup.submenu(9, "sm_popup_stats_score_sheet_%s" % (page))

			popup.addline(" ")

			popup.addline("0. %s" % (popup_common("cancel")))
			popup.submenu(10, "sm_popup_stats")

			if result <= length:
				popup = popuplib.create("sm_popup_stats_score_sheet_%s" % (page))

				popup.addline("SOCCER MOD %s (%s/%s)" % (popup_text("title"), page, number_pages))
				popup.addline(" ")

def reset_score_sheet():
	es.set("sm_stats_score_sheet_number", 1)

	global score_sheet
	score_sheet = []

	delete_score_sheet()

def score_sheet_add(scorer, assister, winner):
	if es.ServerVar("sm_match_started") == 1:
		minutes = es.ServerVar("sm_match_minutes")
		seconds = es.ServerVar("sm_match_seconds")

		if seconds < 10:
			line = "%s:0%s " % (minutes, seconds)
		else:
			line = "%s:%s " % (minutes, seconds)

		score_t = es.ServerVar("sm_match_score_t")
		score_ct = es.ServerVar("sm_match_score_ct")

		if es.ServerVar("sm_match_swap_names_on") == 0:
			line += "%s - %s " % (score_t, score_ct)
		else:
			line += "%s - %s " % (score_ct, score_t)

		team = es.ServerVar("sm_stats_shooter_team")

		line += scorer.name
		line += " "

		if scorer.team == winner:
			if int(assister) > 0:
				if assister.team == winner:
					line += "(%s)" % (assister.name)
				else:
					line += "(%s)" % (popup_text("assist"))
			else:
				line += "(%s)" % (popup_text("assist"))
		else:
			line += "(%s)" % (popup_text("own_goal"))

		score_sheet.append(line)

		delete_score_sheet()
		create_score_sheet()

def score_sheet_console(userid, choice, popupid):
	cmd = "echo ********************"

	es.cexec(userid, cmd)
	es.cexec(userid, "echo SOCCER MOD %s" % (popup_text("title")))
	es.cexec(userid, cmd)

	for row in score_sheet:
		es.cexec(userid, "echo %s" % (row))

	es.cexec(userid, cmd)

	popuplib.send(popupid, userid)

def delete_score_sheet():
	number = 1
	exists = 1
	while exists == 1:
		name = "sm_popup_stats_score_sheet_%s" % (number)
		if popuplib.exists(name):
			popuplib.delete(name)
		else:
			exists = 0
		number += 1

# ******************** Unload ********************
def unload():
	es.unload("soccer_mod_2011/modules/stats/ranking")

	es.keygroupdelete("soccer_mod_stats")
	es.keygroupdelete("soccer_mod_time")

	popuplib.delete("sm_popup_stats")
	popuplib.delete("sm_popup_stats_team")
	delete_score_sheet()

	es.unregclientcmd(es.ServerVar("sm_stats_event"))

	connection.commit()
	connection.close()