﻿import es
import gamethread
import keyvalues
import langlib
import playerlib
import popuplib
import random
import vecmath

lang_dir = "cstrike/addons/eventscripts/soccer_mod_2011/language/"

messages = langlib.Strings(lang_dir + "modules/training/messages_training.ini")
messages_common = langlib.Strings(lang_dir + "messages_common.ini")
texts = langlib.Strings(lang_dir + "modules/training/popup_training.ini")
common = langlib.Strings(lang_dir + "popup_common.ini")

settings = ["randomness", "fire_rate", "height", "power"]

# ******************** Load ********************
def load():
	create_popup()

	es.keygroupcreate("soccer_mod_training")
	es.keygroupcreate("sm_keygroup_cannon")

	es.set("sm_training_spawn_delay", 0)
	es.set("sm_training_position_delay", 0)
	es.set("sm_training_goals_disabled", 0)

	es.addons.registerSayFilter(filter)

	es.load("soccer_mod_2011/modules/training/cannon")

# ******************** Create popup ********************
def create_popup():
	text = langlib.Strings(lang_dir + "modules/training/popup_training.ini")

	popup = popuplib.create("sm_popup_admin_training")

	popup.addline("SOCCER MOD %s" % (text("title")))
	popup.addline(" ")
	popup.addline("->1. %s" % (text("cannon")))
	popup.addline("->2. %s" % (text("spawn")))
	popup.addline("->3. %s" % (text("disable_goals")))
	popup.addline("->4. %s" % (text("enable_goals")))
	popup.addline("->5. Personal cannon")
	popup.addline(" ")
	popup.addline("->9. %s" % (common("back")))
	popup.addline(" ")
	popup.addline("->0. %s" % (common("close")))

	popup.submenu(1, "sm_popup_admin_training_cannon")
	popup.select(2, popup_choice)
	popup.select(3, popup_choice)
	popup.select(4, popup_choice)
	popup.select(5, personal)

	for i in range(6, 9):
		popup.submenu(i, "sm_popup_admin_training")

	popup.submenu(9, "sm_popup_admin")

# ******************** Events ********************
def es_map_start(event_var):
	es.set("sm_training_goals_disabled", 0)

def round_start(event_var):
	if es.ServerVar("soccer_mod_allowed") == 1:
		delete_keygroup()
		es.keygroupcreate("soccer_mod_training")

		if es.ServerVar("sm_training_goals_disabled") == 1:
			players = playerlib.getUseridList("#all")
			if len(players) > 0:
				es.fire(random.choice(players), "trigger_hurt", "Kill")

		cannon = keyvalues.getKeyGroup("sm_keygroup_cannon")
		for item in cannon:
			if int(es.keygetvalue("sm_keygroup_cannon", item, "enabled")) == 1:
				es.keysetvalue("sm_keygroup_cannon", item, "enabled", 0)

def player_disconnect(event_var):
	if es.ServerVar("soccer_mod_allowed") == 1:
		if len(es.getUseridList()) < 1:
			es.set("sm_training_goals_disabled", 0)

		if es.exists("key", "sm_keygroup_cannon", event_var["es_steamid"]) == 1:
			es.keydelete("sm_keygroup_cannon", event_var["es_steamid"])

			players = playerlib.getUseridList("#all")
			if len(players) > 0:
				es.fire(random.choice(players), "sm_training_%s_personal" % (event_var["userid"]), "Kill")

def filter(userid, text, teamonly):
	steamid = es.getplayersteamid(userid)
	if es.exists("key", "sm_keygroup_set", steamid) == 1:
		text = text.strip('"')
		type = es.keygetvalue("sm_keygroup_set", steamid, "setting_type")
		if type == "training_personal_randomness":
			set_value(userid, text, "randomness", 0, 1, 500)
			return (0, None, 0)
		elif type == "training_personal_fire_rate":
			set_value(userid, text, "fire_rate", 1, 0.5, 10)
			return (0, None, 0)
		elif type == "training_personal_height":
			set_value(userid, text, "height", 0, 1, 500)
			return (0, None, 0)
		elif type == "training_personal_power":
			set_value(userid, text, "power", 3, 0.001, 10000)
			return (0, None, 0)
		else:
			return (userid, text, teamonly)
	else:
		return (userid, text, teamonly)

# ******************** Commands ********************
def cmd_spawn():
	tokens = {}
	userid = es.getcmduserid()
	player = playerlib.getPlayer(userid)

	if es.ServerVar("soccer_mod_allowed") == 1:
		if es.exists("key", "soccer_mod_admins", player.steamid) == 1:
			try:
				if int(es.keygetvalue("soccer_mod_admins", player.steamid, "training")) == 1:
					popup_choice(userid, 2, "none")
				else:
					es.tell(userid, "#green", messages_common("not_allowed_command", tokens, player.get("lang")))
			except ValueError:
				es.tell(userid, "#green", messages_common("not_allowed_command", tokens, player.get("lang")))
	else:
		es.tell(userid, "#green", messages_common("not_allowed", tokens, player.get("lang")))

# ******************** Popup choice ********************
def popup_choice(userid, choice, popupid):
	tokens = {}
	player = playerlib.getPlayer(userid)

	if es.ServerVar("sm_match_started") == 0:
		if choice == 2:
			if es.exists("key", "soccer_mod_training", player.steamid) == 0:
				if es.ServerVar("sm_training_spawn_delay") == 0:
					es.set("sm_training_spawn_delay", 1)
					gamethread.delayed(0.1, es.set, ("sm_training_spawn_delay", 0))

					es.server.queuecmd("es_xprop_physics_create %s %s" % (userid, es.ServerVar("sm_training_model_ball")))

					gamethread.delayed(0.01, create_key, (userid, player.steamid))

					es.tell(userid, "#green", messages("spawn", tokens, player.get("lang")))
				else:
					es.tell(userid, "#green", "You have to wait 0.1 seconds before you can spawn a ball.")
			else:
				es.fire(userid, "sm_training_%s" % (userid), "Kill")

				es.keydelete("soccer_mod_training", player.steamid)

				es.tell(userid, "#green", messages("remove", tokens, player.get("lang")))

		elif choice == 3:
			if es.ServerVar("sm_training_goals_disabled") == 0:
				es.server.queuecmd("es_xfire %s trigger_hurt Kill" % (userid))

				es.set("sm_training_goals_disabled", 1)

				tokens["name"] = player.name
				lang_msg("disabled", tokens)
			else:
				es.tell(userid, "#green", messages("already_disabled", tokens, player.get("lang")))

		elif choice == 4:
			if es.ServerVar("sm_training_goals_disabled") == 1:
				es.server.queuecmd("mp_restartgame 1")

				es.set("sm_training_goals_disabled", 0)

				tokens["name"] = player.name
				lang_msg("enabled", tokens)
			else:
				es.tell(userid, "#green", messages("already_enabled", tokens, player.get("lang")))
	else:
		es.tell(userid, "#green", messages_common("no_option_match", tokens, player.get("lang")))

	if popupid != "none":
		popuplib.send("sm_popup_admin_training", userid)

def personal(userid, choice, popupid):
	name = "sm_popup_%s" % (userid)
	if popuplib.exists(name):
		popuplib.delete(name)

	player = playerlib.getPlayer(userid)

	popup = popuplib.create(name)

	popup.addline("SOCCER MOD %s" % (texts("title")))
	popup.addline(" ")

	values = ["position", "aim", "on", "off"]
	for i in range(0, len(values)):
		popup.addline("->%s. %s" % (i + 1, texts(values[i])))

	if es.exists("key", "sm_keygroup_cannon", player.steamid) == 0:
		es.keycreate("sm_keygroup_cannon", player.steamid)
		es.keysetvalue("sm_keygroup_cannon", player.steamid, "index", 0)
		es.keysetvalue("sm_keygroup_cannon", player.steamid, "position", "0,0,15")
		es.keysetvalue("sm_keygroup_cannon", player.steamid, "aim_position", "0,0,15")
		es.keysetvalue("sm_keygroup_cannon", player.steamid, "enabled", 0)
		es.keysetvalue("sm_keygroup_cannon", player.steamid, "randomness", 1)
		es.keysetvalue("sm_keygroup_cannon", player.steamid, "fire_rate", 2.5)
		es.keysetvalue("sm_keygroup_cannon", player.steamid, "height", 1)
		es.keysetvalue("sm_keygroup_cannon", player.steamid, "power", 10000.0)

	for i in range(0, len(settings)):
		popup.addline("->%s. %s: %s" % (i + 5, texts(settings[i]), es.keygetvalue("sm_keygroup_cannon", player.steamid, settings[i])))

	popup.addline(" ")
	popup.addline("->9. %s" % (common("back")))
	popup.addline(" ")
	popup.addline("->0. %s" % (common("close")))

	for i in range(1, 9):
		popup.select(i, personal_choice)

	popup.submenu(9, "sm_popup_admin_training")

	popuplib.send(name, userid)

def personal_choice(userid, choice, popupid):
	tokens = {}
	player = playerlib.getPlayer(userid)

	if es.ServerVar("sm_match_started") == 0:
		if choice == 1:
			if es.ServerVar("sm_training_position_delay") == 0:
				es.set("sm_training_position_delay", 1)
				gamethread.delayed(0.1, es.set, ("sm_training_position_delay", 0))

				es.set("x", 0)
				es.set("y", 0)
				es.set("z", 0)

				es.server.queuecmd("playerget viewcoord %s x y z" % (userid))

				gamethread.delayed(0.01, set_position, (userid, player.steamid))

				es.tell(userid, "#green", "You have set the cannon position.")
			else:
				es.tell(userid, "#green", "You have to wait 0.1 seconds before you can set your cannon.")
		elif choice == 2:
			if es.ServerVar("sm_training_position_delay") == 0:
				es.set("sm_training_position_delay", 1)
				gamethread.delayed(0.1, es.set, ("sm_training_position_delay", 0))

				es.set("x", 0)
				es.set("y", 0)
				es.set("z", 0)

				es.server.queuecmd("playerget viewcoord %s x y z" % (userid))

				gamethread.delayed(0.01, set_aim_position, (userid, player.steamid))

				es.tell(userid, "#green", "You have set the cannon aim position.")
			else:
				es.tell(userid, "#green", "You have to wait 0.1 seconds before you can set your cannon.")
		elif choice == 3:
			if int(es.keygetvalue("sm_keygroup_cannon", player.steamid, "enabled")) == 0:
				index = es.createentity("prop_physics")

				es.entitysetvalue(index, "targetname", "sm_training_%s_personal" % (userid))
				es.entitysetvalue(index, "model", "models/" + es.ServerVar("sm_training_model_ball"))
				postion = es.keygetvalue("sm_keygroup_cannon", player.steamid, "position").split(",")
				es.entitysetvalue(index, "origin", "%s %s %s" % (postion[0], postion[1], float(postion[2]) + int(es.keygetvalue("sm_keygroup_cannon", player.steamid, "height"))))

				es.keysetvalue("sm_keygroup_cannon", player.steamid, "index", index)
				es.keysetvalue("sm_keygroup_cannon", player.steamid, "enabled", 1)

				es.server.queuecmd("es_xspawnentity %s" % (index))

				shoot(userid, index)

				es.tell(userid, "#green", "You have turned the cannon on.")
			else:
				es.tell(userid, "#green", messages("already_on", tokens, player.get("lang")))

		elif choice == 4:
			if int(es.keygetvalue("sm_keygroup_cannon", player.steamid, "enabled")) == 1:
				es.fire(userid, "sm_training_%s_personal" % (userid), "Kill")

				es.keysetvalue("sm_keygroup_cannon", player.steamid, "enabled", 0)

				gamethread.cancelDelayed("sm_training_%s" % (userid))

				es.tell(userid, "#green", "You have turned the cannon off.")
			else:
				es.tell(userid, "#green", messages("already_off", tokens, player.get("lang")))

		elif choice > 4:
			if es.exists("key", "sm_keygroup_set", player.steamid) == 0:
				es.keycreate("sm_keygroup_set", player.steamid)

			es.keysetvalue("sm_keygroup_set", player.steamid, "setting_type", "training_personal_%s" % (settings[choice - 5]))
			es.tell(userid, "#green", messages("type_%s" % (settings[choice - 5]), tokens, player.get("lang")))

		if choice < 5:
			popuplib.send(popupid, userid)
	else:
		es.tell(userid, "#green", messages_common("no_option_match", tokens, player.get("lang")))
		popuplib.send(popupid, userid)

# ******************** Functions ********************
def lang_msg(text, tokens):
	for player in playerlib.getPlayerList("#human"):
		es.tell(player.userid, "#green", messages(text, tokens, player.get("lang")))

def create_key(userid, steamid):
	es.keycreate("soccer_mod_training", steamid)
	es.keysetvalue("soccer_mod_training", steamid, "ball_index", es.ServerVar("eventscripts_lastgive"))

	es.entitysetvalue(es.ServerVar("eventscripts_lastgive"), "targetname", "sm_training_%s" % (userid))

def set_position(userid, steamid):
	es.keysetvalue("sm_keygroup_cannon", steamid, "position", "%s,%s,%s" % (es.ServerVar("x"), es.ServerVar("y"), float(es.ServerVar("z")) + 15))
	show_lines(es.ServerVar("x"), es.ServerVar("y"), es.ServerVar("z"))

def set_aim_position(userid, steamid):
	es.keysetvalue("sm_keygroup_cannon", steamid, "aim_position", "%s,%s,%s" % (es.ServerVar("x"), es.ServerVar("y"), es.ServerVar("z")))
	show_lines(es.ServerVar("x"), es.ServerVar("y"), es.ServerVar("z"))

def shoot(userid, index):
	if es.exists("userid", userid) == 1:
		player = playerlib.getPlayer(userid)

		if es.exists("key", "sm_keygroup_cannon", player.steamid) == 1:
			if int(es.keygetvalue("sm_keygroup_cannon", player.steamid, "enabled")) == 1:
				postion = es.keygetvalue("sm_keygroup_cannon", player.steamid, "position").split(",")
				aim_postion = es.keygetvalue("sm_keygroup_cannon", player.steamid, "aim_position").split(",")

				es.server.queuecmd("es_xphysics active teleport %s %s,%s,%s" % (index, postion[0], postion[1], float(postion[2]) + int(es.keygetvalue("sm_keygroup_cannon", player.steamid, "height"))))

				aim_x = float(aim_postion[0])
				aim_y = float(aim_postion[1])
				aim_z = float(aim_postion[2])

				randomness = int(es.keygetvalue("sm_keygroup_cannon", player.steamid, "randomness"))
				if randomness > 0:
					aim_x = random.uniform(aim_x - randomness, aim_x + randomness)
					aim_y = random.uniform(aim_y - randomness, aim_y + randomness)
					aim_z = random.uniform(aim_z - randomness, aim_z + randomness)

				vector1 = vecmath.vector([aim_x, aim_y, aim_z])

				vector2 = vecmath.vector([float(postion[0]), float(postion[1]), float(postion[2])])

				aim = (vector1 - vector2) * float(es.keygetvalue("sm_keygroup_cannon", player.steamid, "power"))
				gamethread.delayed(0.1, es.server.queuecmd, "es_xphysics active setvelocity %s %s" % (index, aim))

				gamethread.delayedname(float(es.keygetvalue("sm_keygroup_cannon", player.steamid, "fire_rate")), "sm_training_%s" % (userid), shoot, (userid, index))

def set_value(userid, text, type, decimals, min, max):
	tokens = {}
	player = playerlib.getPlayer(userid)

	if text != "!cancel":
		error = messages("type_%s" % (type), tokens, player.get("lang"))
		try:
			if decimals < 1:
				value = int(round(float(text), 0))
			else:
				value = round(float(text), decimals)

			if value >= min:
				if value <= max:
					es.keysetvalue("sm_keygroup_cannon", player.steamid, type, value)

					es.keydelete("sm_keygroup_set", player.steamid)

					personal(userid, 1, "None")
				else:
					es.tell(userid, "#green", error)
			else:
				es.tell(userid, "#green", error)
		except ValueError:
			es.tell(userid, "#green", error)
	else:
		es.keydelete("sm_keygroup_set", player.steamid)

		personal(userid, 1, "None")

def show_lines(x, y, z):
	model = es.precachemodel("sprites/crystal_beam1.vmt")

	pos1 = "%s,%s,%s" % ((x - 25), y, z)
	pos2 = "%s,%s,%s" % ((x + 25), y, z)
	es.server.queuecmd("es_effect beam %s %s %s %s 0 0 5 5 5 0 1 255 0 0 255 5" % (pos1, pos2, model, model))

	pos1 = "%s,%s,%s" % (x, (y - 25), z)
	pos2 = "%s,%s,%s" % (x, (y + 25), z)
	es.server.queuecmd("es_effect beam %s %s %s %s 0 0 5 5 5 0 1 255 0 0 255 5" % (pos1, pos2, model, model))

	pos1 = "%s,%s,%s" % (x, y, (z - 25))
	pos2 = "%s,%s,%s" % (x, y, (z + 25))
	es.server.queuecmd("es_effect beam %s %s %s %s 0 0 5 5 5 0 1 255 0 0 255 5" % (pos1, pos2, model, model))

def delete_keygroup():
	if es.exists("keygroup", "soccer_mod_training") == 1:
		es.keygroupdelete("soccer_mod_training")

# ******************** Unload ********************
def unload():
	delete_keygroup()
	es.keygroupdelete("sm_keygroup_cannon")

	popuplib.delete("sm_popup_admin_training")

	es.addons.unregisterSayFilter(filter)

	es.unload("soccer_mod_2011/modules/training/cannon")